import linuxcnc
import os
import hal
import hal_glib
import gtk
import popupkeyboard
from math import pi, cos, sin, tan, sqrt, ceil, radians
import operator
import hal_glib

s = linuxcnc.stat()
c = linuxcnc.command()

initvals = {'teeth':20, 'module':2, 'dp':12.7, 'cpmm':1.570796, 'cpin':.061842, 'shift':0, 'helix':0, 'pa':20, 'spindle':0, 
	'feed':0.15, 'distance':12.7, 'cut':2, 'finish':0.1, 'end':20}
increment = {'teeth':1, 'module':0.1, 'dp':1, 'cpmm':1, 'cpin':0.125, 'shift':.05, 'helix':1, 'pa':5.5, 'spindle':10, 
	'feed':0.01, 'distance':1, 'cut':.5, 'finish':0.01, 'end':1}
maximum = {'teeth':500, 'module':10, 'dp':50, 'cpmm':10, 'cpin':1, 'shift':1, 'helix':60, 'pa':20, 'spindle':500,
	 'feed':1, 'distance':1000, 'cut':20, 'finish':1, 'end':1000}
minimum = {'teeth':1, 'module':.05, 'dp':2, 'cpmm':0, 'cpin':0, 'shift':-1, 'helix':-60, 'pa':14.5, 'spindle':0, 
	'feed':0, 'distance':0, 'cut':0, 'finish':0, 'end':0}

def ok_for_mdi():
    s.poll()
    return not s.estop and s.enabled and s.homed and (s.interp_state == linuxcnc.INTERP_IDLE)

def recalc(self, label):
	v = float(label.get_text())
	teeth = float(self.builder.get_object("teeth").get_text())
	pa = float(self.builder.get_object("pa").get_text())
	shift = float(self.builder.get_object("shift").get_text())

	flag = 1
	if label.get_name() in ["teeth", 'pa', 'shift']: # use module ignore others
		mod = float(self.builder.get_object("module").get_text())
		self.teeth_out.set(teeth)
		flag = 0
	elif label.get_name() == "dp":
		dp = v
		mod = 25.4 / v
		cpmm = 25.4 * pi / v
		cpin = pi / v
	elif label.get_name() == "module":
		dp = 25.4 / v
		mod = v
		cpmm = pi / v
		cpin = pi  / (v * 25.4)
	elif label.get_name() == "cpmm":
		dp = pi * 25.4 / v
		mod = v / pi
		cpmm = v
		cpin = v / 25.4
	elif label.get_name() == "cpin":
		dp = pi / v
		mod = 25.4 * v / pi
		cpmm = v * 25.4
		cpin = v
	else:
		return

	if flag:
		self.builder.get_object("module").set_text(str(mod))
		self.builder.get_object("dp").set_text(str(dp))
		self.builder.get_object("cpmm").set_text(str(cpmm))
		self.builder.get_object("cpin").set_text(str(cpin))

# http://www.sdp-si.com/D805/D805_PDFS/Technical/8050T081.pdf

	pcd = (teeth + 2 * shift) * mod
	od = pcd + 2 * mod
	depth = 2.4 * mod if mod < 1 else 2.25 * mod
	root = od - depth * 2
	
	f = shift / teeth
	pa = radians(pa)
	Kf = (1/pi)*((1/cos(pa))*sqrt((1+2*f)**2-(cos(pa))**2)-(tan(pa)-pa)-(2*f*tan(pa)))

	Zmth = teeth * Kf + 0.5
	count =  round(Zmth)
	span = mod * cos(pa) *(pi * (count - 0.5) + teeth * (tan(pa)-pa)) + 2 * shift * mod * sin(pa)

	self.builder.get_object('pcdO').set_text(str(pcd))
	self.builder.get_object('odO').set_text(str(od))
	self.builder.get_object('depthO').set_text(str(depth))
	self.builder.get_object('rootO').set_text(str(root))
	self.builder.get_object('spanO').set_text(str(span))
	self.builder.get_object('countO').set_text(str(count))
	self.builder.get_object('end').set_text(str(root/2))


class HandlerClass :

	def entry_click(self, label, event=None, mystery=None):
		print gtk.Buildable.get_name(label)
		print event
		g_popkbd = popupkeyboard.PopupKeyboard(use_coord_buttons=False)
		if g_popkbd.run(initial_value='',title='your_title'):
			label.set_text(str(g_popkbd.get_result()))
			recalc(self, label)
		else:
			print "user canceled"

	def inc_button(self, label, data=None, mystery=None):
		val = float(label.get_text())
		name = gtk.Buildable.get_name(label)
		val = val + increment[name]
		if val > maximum[name]:
			val = maximum[name]
		elif val < minimum[name]:
			val = minimum[name]
		label.set_text(str(val))
		recalc(self, label)

	def dec_button(self, label, data=None, mystery=None):
		val = float(label.get_text())
		name = gtk.Buildable.get_name(label)
		val = val - increment[name]
		if val > maximum[name]:
			val = maximum[name]
		elif val < minimum[name]:
			val = minimum[name]
		label.set_text(str(val))
		recalc(self, label)

	def spindle_start(self, label, data=None):
		if ok_for_mdi():
			s = float(self.builder.get_object('spindle').get_text())
			c.mode(linuxcnc.MODE_MDI)
			c.wait_complete() # wait until mode switch executed
			if label.get_name() == 'spindle_anti':
				c.mdi('M4S%f'%s)
			else:
				c.mdi('M3S%f'%s)
				
	def spindle_changed(self, label, data=None):
		if ok_for_mdi():
			s = float(self.builder.get_object("spindle").get_text())
			c.mode(linuxcnc.MODE_MDI)
			c.wait_complete() # wait until mode switch executed
			c.mdi("S%f"%s)
			
	def cycle_start(self, label, data=None):
		feed = float(self.builder.get_object("feed").get_text()) / float(self.builder.get_object("teeth").get_text())
		distance = float(self.builder.get_object("distance").get_text())
		cut = float(self.builder.get_object("cut").get_text())
		finish = float(self.builder.get_object("finish").get_text())
		end = float(self.builder.get_object("end").get_text())
		if label.get_name() == 'FeedLeft':
			 distance = -distance
		if ok_for_mdi():
   			c.mode(linuxcnc.MODE_MDI)
   			c.wait_complete() # wait until mode switch executed
			c.mdi("o <hobbing> call [%f] [%f] [%f] [%f] [%f]" % (feed, distance, cut, finish, end))
		else:
			print "Can't enter MDI mode"
			
	def __init__(self, halcomp, builder, useropts):
		self.builder = builder
		self.teeth_out = hal_glib.GPin(halcomp.newpin('teeth', hal.HAL_FLOAT, hal.HAL_OUT))
		self.teeth_out.set(20)
		for l in initvals.keys():
			w = self.builder.get_object(l)
			w.set_text(str(initvals[l]))

def get_handlers(halcomp,builder,useropts):
	return [HandlerClass(halcomp,builder,useropts)]
